/**************************************************************

File Name:	Single_Sample_FIR.asm

Date Modified:	12/10/99	GGL
				05/01/00	RFG
				09/19/04	JDT

Description:
			Subroutine that implements a Single Sample FIR Filter given
			coefficients and samples.

			Equation:	y(n) = Summation from k=0 to M of h(k)*x(n-k)

Calling Parameters:
			b0,i0 = address of delay line buffer
			l0 = length of delay line buffer - 1
			b8,i8 = address of coefficients buffer
			l8 = length of coefficients buffer
			r1 = number of taps in the filter
			m1 = 1
			m2 = -1
			m3,m9 = 2

Assumptions:
			All arrays must start on even address boundaries.
			All arrays must have an even number length (zero  
		    pad if necessary)
			Delay Line must be of length TAPS + 1
			Delay line and Input buffers stored in Block 1.
			Instructions, Output, and Coefficients stored in Block 0.

Return Values:
			f8 = result 

Registers Affected:
			r0,s0,r4,s4,r8,s8,r12,s12
			i0,i8

Cycle Count:
			TAPS/2 + 12 + 4 cache misses 


Memory Usage:
			Instructions Words (48-bits):
				16 instruction words
			
			Data Words (16, 32, 40, or 64-bits):
				Number of taps locations for coefficients (32-bits)
				Number of taps + 1 locations for the delay line buffer (32-bits)
				Number of samples locations for the input buffer (32-bits)
				Number of samples locations for the output buffer (32-bits)

Notes: 
			Silicon Revision Issue:
			This code will not function properly on silicon revision 0.0 
			due to an anomaly related to internal addressing in SIMD mode.
			For more information please refer to the anomaly list.

			Circular Buffering Issue:
			Because SIMD or Long word access transfer two 32-bit words, programs
			must be careful when using these accesses in circular buffering.
			It is important that SIMD or Long word accesses do not cross a circular
		 	buffer boundary.  If a SIMD	mode access occurs using a circular buffer
			index register that points to the last location in the circular buffer
		 	(end of buffer), the resulting access transfers the last location in
		 	the circular buffer and the	first location outside the buffer (end of buffer + 1).

**************************************************************/

#include    "def21364.h"				/* Symbol Definition File */

.global ss_fir;

/* program memory code */
.section/pm seg_pmco;

ss_fir:
	bit set MODE1 CBUFEN;				/* Circular Buffer Enable, one cycle effect latency */
	nop;								/* Circular Buffering not in effect until next cycle */
	
	s0 = dm(i0, m1);					/* move pointer to delay[1] */
	
	bit set MODE1 PEYEN;				/* SIMD Mode Enable, one cycle effect latency */
	s0 = dm(i0, m2);					/* load s0 with the value of delay[1] for SIMD store, move pointer to delay[0] */
	
	dm(i0,m3)=f0, f4 = pm(i8,m9);		/* transfer sample to delayline, done in SIMD to load end of buffer + 1 */
										/* to compensate for circular buffer issue described above, read 2 coeffs */										

		f8=f0*f4, f0=dm(i0,m3), f4=pm(i8,m9);	/* samples * coeffs, read 2 samples, read 2 coeffs */
		f12=f0*f4, f0=dm(i0,m3), f4=pm(i8,m9);	/* samples * coeffs, read 2 samples, read 2 coeffs */
        lcntr=r3, do macs until lce;			/* FIR loop */
macs:       f12=f0*f4, f8=f8+f12, f0=dm(i0,m3), f4=pm(i8,m9);	/* samples * coeffs, accum, read 2 samples, read 2 coeffs */
        f12=f0*f4, f8=f8+f12, s0=dm(i0,m2);		/* samples * coeffs, accum, dummy read to move pointer to oldest sample */
        f8=f8+f12;								/* final SIMD accum */
        r12=s8;									/* move PEy total into PEx register file */
		
	rts (db);
	bit clr MODE1 CBUFEN | PEYEN;		/* Circular Buffer Disable, SIMD Mode Disable */
	f8=f8+f12;							/* last accum */
